#ifndef ESP32_API_HPP
#define ESP32_API_HPP

#ifdef ESP32
#include "WebServer.h"
#elif defined(ESP8266)
#include "ESP8266WebServerSecure.h"
#endif

#include "uri/UriRegex.h"

#include "Rules.hpp"
#include "Utils.hpp"

namespace fw
{
    class API : public Rules
    {
    private:
#ifdef ESP32
        WebServer *server;
#elif defined(ESP8266)
        BearSSL::ESP8266WebServerSecure *server;
        BearSSL::ServerSessions *serverCache;
#endif
        credential_t credentials;

        ok_t setup_auth(const char *, const char *);
        auth_t check_auth();

        void setup_routing(const char *, const char *);
        void get_firewall_rule_handler();
        void get_firewall_rules_handler();
        void post_firewall_handler();
        void delete_firewall_handler();
        void not_found_handler();

        bool request_has_firewall_parameter();
        String json_new_attribute(String key, String value, bool last = false);
        String json_new_attribute(String key, uint8_t value, bool last = false);
        void json_generic_response(String serialized_string, const uint16_t response_code);
        void json_message_response(String message, const uint16_t response_code);
        String construct_json_firewall_rule(firewall_rule_t *);
        String construct_json_firewall();

    protected:
        void handle_client();

    public:
        API(const char *cert, const char *key, const char *username, const char *password, const uint16_t port);
        ~API();
    };
}

#endif
